"""
    Database module
"""

# Modules XBMC
import os, sys, re, xml.dom.minidom
import xbmc, xbmcvfs

# Modules Custom
import jsonrpc
from log import log

# Import SQLite for local database
try:
    from sqlite3 import dbapi2 as sqlite
except:
    from pysqlite2 import dbapi2 as sqlite

# Import MySQL for shared database
import mysql.connector
    
VIDEO_FIELDS_MOVIESET = [ "title", "fanart", "thumbnail" ]
VIDEO_FIELDS_MOVIE = [ "title", "fanart", "thumbnail", "set", "file" ]
SORTTITLE = { "method": "sorttitle", "order": "ascending" }
ARTWORK_TYPE_THUMB = "thumb"
ARTWORK_TYPE_POSTER = "poster"
ARTWORK_TYPE_FANART = "fanart"
ARTWORK_TYPE_LOGO = "clearlogo"
ARTWORK_TYPE_CLEARART = "clearart"
ARTWORK_TYPE_BANNER = "banner"
DATABASE_VERSION_FRODO = "75"
DATABASE_VERSION_GOTHAM = "76"

# Class to implement a custom cursor returning dictionary results
class MySQLCursorDict(mysql.connector.cursor.MySQLCursor):
    
    def _row_to_python(self, rowdata, desc=None):
        row = super(MySQLCursorDict, self)._row_to_python(rowdata, desc)
        if row:
            return dict(zip(self.column_names, row))
        return None

# Video database class performing fetching and updating operations
class Database():
    
    def initialise( self, dbOnly ):
        self.dbOnly = dbOnly
        if (not dbOnly):
            self.json_api = jsonrpc.jsonrpcAPI()

        self.useMySQL = False
        myVideoConfig = self.getVideoDBConfig()
        dbType = myVideoConfig["type"]
        dbName = myVideoConfig["name"]
        
        if (dbType == "mysql"):
            log( "Using MySQL connector: %s" % myVideoConfig )
            self.useMySQL = True
            self.db_conn = mysql.connector.connect(
                              user=myVideoConfig["user"], password=myVideoConfig["pass"],
                              host=myVideoConfig["host"], port=myVideoConfig["port"],
                              database=dbName.encode("utf-8"))
        else:
            log( "Using SQlite connector: %s" % myVideoConfig )
            dbName = "%s.db" % dbName
            self.video_db = os.path.join(xbmc.translatePath("special://database"), dbName)
            self.db_conn = sqlite.connect(self.video_db)
        self.db_conn.row_factory = sqlite.Row

    def getElementText(self, domItem, name, default=""):
        element = domItem.getElementsByTagName(name)
        if ( len(element) > 0 ):
            return element[0].firstChild.data
        return default
        
    def getVideoDBConfig(self):
        result = {}
        
        # Set default to sqlite
        result["type"] = "sqlite3"
        result["name"] = "MyVideos"
        
        # Look for overrides in advanced settings
        advanced_xml = xbmc.translatePath( "special://profile/advancedsettings.xml" )
        if xbmcvfs.exists( advanced_xml ):
            log( "Parsing advancedsettings.xml" )
            if hasattr( xbmcvfs, "File" ):
                OPEN = xbmcvfs.File
            else:
                OPEN = open
            configFile = OPEN( advanced_xml )

            # Parse XML
            try:
                document = xml.dom.minidom.parse(configFile)
            except:
                print "Error parsing XML: ", sys.exc_info()
                return result
            for item in document.getElementsByTagName("videodatabase"):
                result["type"] = self.getElementText(item, "type", "sqlite3")
                result["name"] = self.getElementText(item, "name", result["name"])
                if (result["type"] == "mysql"):
                    result["host"] = self.getElementText(item, "host")
                    result["port"] = self.getElementText(item, "port", "3306")
                    result["user"] = self.getElementText(item, "user")
                    result["pass"] = self.getElementText(item, "pass")
            configFile.close()
            
        # Check the XBMC version to determine the video db name
        xbmcVersion = xbmc.getInfoLabel( "system.buildversion" )
        log( "XBMC build version: %s" % xbmcVersion )
        if xbmcVersion.find("13") == 0:
            result["name"] = "%s%s" % (result["name"], DATABASE_VERSION_GOTHAM)
            log( "Using Gotham DB version: %s" % result["name"] )
        else:
            result["name"] = "%s%s" % (result["name"], DATABASE_VERSION_FRODO)
            log( "Using Frodo DB version: %s" % result["name"] )
            
        return result
            
    def getCursor(self):
        if (self.useMySQL):
            return self.db_conn.cursor(cursor_class=MySQLCursorDict)
        else:
            return self.db_conn.cursor()
            
    def executeQuery(self, cursor, query, params=None):
        if (self.useMySQL):
            query = query.replace("?", "%s")
            
        if (params != None):
            return cursor.execute(query, params)
        else:
            return cursor.execute(query)
            
    def getMovieSets(self):
        if (self.dbOnly):
            cur = self.getCursor()
            self.executeQuery(cur, "SELECT * FROM sets")
            rows = cur.fetchall()
            movie_sets = []
            for row in rows:
                movie_set = {}
                movie_set[ "setid" ] = row[ "idSet" ]
                movie_set[ "label" ] = row[ "strSet" ]
                movie_sets.append(movie_set)
            return movie_sets
        else:
            json = self.json_api.VideoLibrary.GetMovieSets( properties=VIDEO_FIELDS_MOVIESET )
            return json.get( "sets", [] )
    
    def getMoviesInSet( self, setId ):
        if (self.dbOnly):
            cur = self.getCursor()
            self.executeQuery(cur, "SELECT * FROM movie where idSet = ?", (setId,))
            rows = cur.fetchall()
            movies = []
            for row in rows:
                movie = {}
                movie[ "title" ] = row[ "c00" ]
                movie[ "file" ] = row[ "c22" ]   # only folder, but enough
                movies.append(movie)
            return movies
        else:
            # GetMovieSetDetails only returns a Video.Fields.MovieSet object, but we
            # need information about movies in the set, so use Files.GetDirectory
            json = self.json_api.Files.GetDirectory( directory="videodb://1/7/%i/" % setId, \
                                               properties=VIDEO_FIELDS_MOVIE, sort=SORTTITLE, media="video" )
            return json.get( "files", [] )
    
    # Method to update the URL assigned for movie set artwork in the video DB
    def updateDatabase(self, setId, filename, art_type, force_write):
        # Check what type is already set
        cur = self.getCursor()
        self.executeQuery(cur, "SELECT type, url FROM art where media_type = ? and media_id = ? and url != ''", ("set", setId))
        existing_type_map = {}
        rows = cur.fetchall()
        for row in rows:
            existing_type_map[row["type"] ] = row["url"]
        
        # Work out what should be updated
        # log ( "Existing artwork for set %s: %s" % (setId, existing_type_map) )
        if ( art_type in existing_type_map ):
            log ( "Existing %s: %s" % (art_type, existing_type_map[art_type]) )
        update_type = art_type

        # Perform create/update
        updated = 0
        cur = self.getCursor()
        if ( update_type in existing_type_map ):
            if ( existing_type_map[update_type] == filename ):
                if (force_write):
                    log( "Removing existing artwork:\nType: %s\nFile: %s" % (update_type, existing_type_map[update_type]), xbmc.LOGDEBUG )
                    self.executeQuery(cur, "DELETE from art where media_type = ? and media_id = ? and type = ?", \
                                 ("set", setId, update_type) )
                    log( "Adding artwork:\nType: %s\nFile: %s" % (update_type, filename), xbmc.LOGDEBUG )
                    self.executeQuery(cur, "INSERT INTO art(media_id, media_type, type, url) VALUES(?,?,?,?)", \
                                 (setId, "set", update_type, filename) )
                    updated += 1
            else:
                log( "Updating artwork:\nType: %s\nExisting: %s\nNew:      %s" % (update_type, existing_type_map[update_type], filename), xbmc.LOGDEBUG )
                self.executeQuery(cur, "UPDATE art SET url = ? where media_type = ? and media_id = ? and type = ?", \
                             (filename, "set", setId, update_type) )
                updated += 1
        else:
            log( "Adding artwork:\nType: %s\nFile: %s" % (update_type, filename), xbmc.LOGDEBUG )
            self.executeQuery(cur, "INSERT INTO art(media_id, media_type, type, url) VALUES(?,?,?,?)", \
                         (setId, "set", update_type, filename) )
            updated += 1
        self.db_conn.commit()
        cur.close()

        return updated
    